@@ -9,7 +9,7 @@ use Encode qw( encode decode );
 use URI;
 use Daizu::Wc;
 use Daizu::Util qw(
-    trim trim_with_empty_null pgregex_escape
+    trim trim_with_empty_null pgregex_escape url_encode url_decode
     parse_db_datetime
     db_select db_select_col db_insert db_update transactionally
     wc_file_data
@@ -297,6 +297,36 @@ sub parent
     return Daizu::File->new($self->{cms}, $self->{parent_id});
 }
 
+=item $file-E<gt>file_at_path($path)
+
+Returns a L<Daizu::File> object for the file found at C<$path> in the
+same working copy as C<$file>.  C<$path> can be an absolute path (if it
+starts S<with '/'>), or it can be relative to the path of C<$file>.
+
+Dies if there is no such file.
+
+=cut
+
+sub file_at_path
+{
+    my ($self, $path) = @_;
+    my $wc_id = $self->{wc_id};
+
+    # The 'daizu' scheme is used for XInclude expansion, so might as well
+    # use it here too.  It doesn't really matter, we just want to get the
+    # URI module to resolve the relative path for us.
+    my $base = 'daizu:///' . url_encode($self->{path});
+    $base .= '/' if $self->{is_dir};
+
+    $path =~ s!/\./!/!g;
+    my $abs_path = URI->new($path)->abs($base)->path;
+    $abs_path =~ s!^/!!;
+    $abs_path = url_decode($abs_path);
+
+    my $wc = Daizu::Wc->new($self->{cms}, $self->{wc_id});
+    return $wc->file_at_path($abs_path);
+}
+
 =item $file-E<gt>issued_at
 
 Return a L<DateTime> object for the publication date and time of the file.
@@ -605,7 +635,15 @@ sub _filter_loaded_article
                 substr($file_path, 0, length $match_path + 1) eq "$match_path/";
 
             my ($object, $method) = @{$plugins->{$match_path}};
-            $doc = $object->$method($cms, $doc);
+            my $result = $object->$method($cms, $self, $doc);
+            die "filter plugin $object->$method didn't return any content"
+                unless defined $result && defined $result->{content};
+
+            $doc = $result->{content};
+            push @{$article->{extra_urls}}, @{$result->{extra_urls}}
+                if defined $result->{extra_urls};
+            push @{$article->{extra_templates}}, @{$result->{extra_templates}}
+                if defined $result->{extra_templates};
 
             # Only execute the best match for each filter.
             next FILTER;
@@ -639,7 +677,7 @@ sub _save_article_content
     $pages_url = '' unless defined $pages_url;
     $self->{article_pages_url} = $pages_url,
 
-    my $xml = $art->{content}->toStringC14N;
+    my $xml = $art->{content}->documentElement->toString;
     croak "article can't be loaded because the result contains a nul byte"
         if $xml =~ /\0/;
     delete $self->{article_doc};
@@ -1128,7 +1166,7 @@ sub authors
     return \@author;
 }
 
-=item $file-E<gt>update_urls_in_db
+=item $file-E<gt>update_urls_in_db([$dup_urls])
 
 Updates the C<url> table to match the current URLs generated by C<$file>,
 as returned by the generator method
@@ -1136,6 +1174,19 @@ L<urls_info()|Daizu::Gen/$gen-E<gt>urls_info($file)>.
 This includes changing active URLs to redirects or marking them 'gone'
 if they are no longer generated by the file.
 
+If this update isn't being done in isolation, but for example is being
+run by the
+L<update_all_file_urls()|Daizu::Util/update_all_file_urls($cms, $wc_id)>
+function, then C<$dup_urls> should be a reference to a hash which can be
+used to keep track of new URLs which cannot be added to the database yet
+because they would conflict with an existing active URL.  This method will
+remove them from the hash later if the original active URL becomes inactive,
+and will replace it in the database with the new one.  Some cleanup code
+is needed in the caller though to finish processing any URLs left in the
+hash, or to signal errors if they are a genuine conflict which cannot be
+resolved.  An error should cause the transaction to update, because items
+in the hash represent unfinished updates to the database.
+
 Returns a list of two values, which can each be either true or false.
 They indicate whether the set of URLs which are redirects or marked as
 'gone' have changed.  The first indicates that at least one redirect has
@@ -1148,19 +1199,23 @@ be regenerated by the caller.
 The work is done in a transaction, so that if it fails there will be
 no changes to the database.
 
+TODO - update docs about return value
+
 =cut
 
 sub update_urls_in_db
 {
-    my ($self) = @_;
-    return transactionally($self->{db}, \&_update_urls_in_db_txn, $self);
+    my ($self, $dup_urls) = @_;
+    return transactionally($self->{db}, \&_update_urls_in_db_txn, $self,
+                           $dup_urls);
 }
 
 sub _update_urls_in_db_txn
 {
-    my ($self) = @_;
+    my ($self, $dup_urls) = @_;
     my $db = $self->{db};
 
+    # Get information about the URLs that we currently have for this file.
     my $sth = $db->prepare(q{
         select *
         from url
@@ -1169,7 +1224,6 @@ sub _update_urls_in_db_txn
     });
     $sth->execute($self->{wc_id}, $self->{guid_id});
 
-    # Get information about the URLs that we currently have for this file.
     my (%old_active, %old_redirect, %old_gone);
     while (my $r = $sth->fetchrow_hashref) {
         my $hash = $r->{status} eq 'A' ? \%old_active :
@@ -1180,8 +1234,18 @@ sub _update_urls_in_db_txn
 
     # Keep track of whether the set of redirects or gone files have changed,
     # which might mean that the caller will need to regenerate some redirect
-    # files.
-    my ($redirects_changed, $gone_changed);
+    # files.  The keys are the filenames which need updating, and the values
+    # are the output configuration hashes where they were found.
+    my (%redirects_changed, %gone_changed);
+
+    # Track changes to URLs for the publishing process.  Each one of these
+    # has the URL as the key and the URL info hash as the value.  For changed
+    # URLs the new URL and URL info is used in the key and value, but the value
+    # also contains a key called 'old_url_info'.
+    # Note that changed URLs are ones where a redirect has been created, and
+    # changes of URL ownership (one file deactivates it and another file
+    # reactivates the same URL) are not recorded.
+    my (%url_activated, %url_deactivated, %url_changed);
 
     # Put the new URLs in the database.  Add the 'id' of each one to the
     # information in @new_url.
@@ -1189,9 +1253,13 @@ sub _update_urls_in_db_txn
     for (@new_url) {
         my $url = $_->{url};
         if (exists $old_active{$url}) {
-            # Was active, and still is.
-            my $id = $_->{id} = $old_active{$url}{id};
-            db_update($db, url => $id,
+            # Was active, and still is.  If a duplicate URL has been created
+            # by another file then at this point we know there's no chance
+            # of it being resolved, so signal an error.
+            croak "new URL '$url' would conflict with existing URL"
+                if defined $dup_urls && exists $dup_urls->{$url};
+            $_->{id} = $old_active{$url}{id};
+            db_update($db, url => $_->{id},
                 method => $_->{method},
                 argument => $_->{argument},
                 content_type => $_->{type},
@@ -1200,28 +1268,32 @@ sub _update_urls_in_db_txn
         }
         elsif (exists $old_redirect{$url}) {
             # Was a redirect, but now active again.
-            my $id = $_->{id} = $old_redirect{$url}{id};
-            db_update($db, url => $id,
+            assert(!defined $dup_urls || !exists $dup_urls->{$url}) if DEBUG;
+            $_->{id} = $old_redirect{$url}{id};
+            db_update($db, url => $_->{id},
                 method => $_->{method},
                 argument => $_->{argument},
                 content_type => $_->{type},
                 status => 'A',
                 redirect_to_id => undef,
             );
+            $url_activated{$url} = $_;
             delete $old_redirect{$url};
-            $redirects_changed = 1;
+            $self->_update_rewrite(\%redirects_changed, 'redirect', $url);
         }
         elsif (exists $old_gone{$url}) {
             # Was gone, but has come back.
-            my $id = $_->{id} = $old_gone{$url}{id};
-            db_update($db, url => $id,
+            assert(!defined $dup_urls || !exists $dup_urls->{$url}) if DEBUG;
+            $_->{id} = $old_gone{$url}{id};
+            db_update($db, url => $_->{id},
                 method => $_->{method},
                 argument => $_->{argument},
                 content_type => $_->{type},
                 status => 'A',
             );
+            $url_activated{$url} = $_;
             delete $old_gone{$url};
-            $gone_changed = 1;
+            $self->_update_rewrite(\%gone_changed, 'gone', $url);
         }
         else {
             # New URL.  It might replace a non-active one belonging to a
@@ -1230,57 +1302,108 @@ sub _update_urls_in_db_txn
                 { wc_id => $self->{wc_id}, url => $url },
                 qw( id status ),
             );
-            if (defined $id) {
+            if (defined $id && !(defined $dup_urls && $status eq 'A')) {
                 if ($status eq 'A') {
                     croak "new URL '$url' would conflict with existing URL";
                 }
                 elsif ($status eq 'R') {
-                    $redirects_changed = 1;
+                    $self->_update_rewrite(\%redirects_changed, 'redirect',
+                                           $url);
                 }
                 elsif ($status eq 'G') {
-                    $gone_changed = 1;
+                    $self->_update_rewrite(\%gone_changed, 'gone', $url);
                 }
+
+                # %$dup_urls should never contain a non-active URL.
+                assert(!defined $dup_urls || !exists $dup_urls->{$url})
+                    if DEBUG;
+
+                # Write the new active URL over the top of an old inactive one.
                 $_->{id} = $id;
-                $_->{id} = db_update($db, url => $id,
-                    guid_id => $self->{guid_id},
-                    method => $_->{method},
-                    argument => $_->{argument},
-                    content_type => $_->{type},
-                    status => 'A',
-                    redirect_to_id => undef,
-                );
-            }
-            else {
-                # This is the only place where new 'url' records are inserted.
-                $_->{id} = db_insert($db, 'url',
-                    url => $url,
-                    wc_id => $self->{wc_id},
+                db_update($db, url => $id,
                     guid_id => $self->{guid_id},
                     generator => $_->{generator},
                     method => $_->{method},
                     argument => $_->{argument},
                     content_type => $_->{type},
                     status => 'A',
+                    redirect_to_id => undef,
                 );
+                $url_activated{$url} = $_;
+            }
+            else {
+                if (defined $id) {
+                    assert(defined $dup_urls) if DEBUG;
+
+                    # We already have one duplicate for this, so it can't
+                    # possibly be resolved by deactivating the existing one.
+                    croak "new URL '$url' would conflict with existing URL"
+                        if exists $dup_urls->{$url};
+
+                    # The new URL is a duplicate of an existing one which is
+                    # still active, but our caller has let it be known that
+                    # there are multiple files being updated at once, so keep
+                    # information about the new URL in the hopes that the old
+                    # one will later be deactivated, allowing it to be replaced
+                    # with this one.
+                    $_->{id} = $id;
+                    $dup_urls->{$url} = {
+                        id => $id,
+                        guid_id => $self->{guid_id},
+                        generator => $_->{generator},
+                        method => $_->{method},
+                        argument => $_->{argument},
+                        type => $_->{type},
+                    };
+                }
+                else {
+                    # This is the only place where new 'url' records are
+                    # inserted.
+                    $_->{id} = db_insert($db, 'url',
+                        url => $url,
+                        wc_id => $self->{wc_id},
+                        guid_id => $self->{guid_id},
+                        generator => $_->{generator},
+                        method => $_->{method},
+                        argument => $_->{argument},
+                        content_type => $_->{type},
+                        status => 'A',
+                    );
+                    $url_activated{$url} = $_;
+                }
             }
         }
     }
 
     # Adjust any previously-active URLs which are no longer active.
     for (values %old_active) {
+        if (defined $dup_urls && exists $dup_urls->{$_->{url}}) {
+            # A new URL which is a duplicate of this one has been created
+            # by a different file, and is waiting in the wings in the hopes
+            # that this file will ditch it.  We can now replace the old one
+            # with the new one from the other file, and thereby resolve
+            # the duplication.
+            my $dup = $dup_urls->{$_->{url}};
+            assert($dup->{id} == $_->{id}) if DEBUG;
+            db_update($db, url => $_->{id},
+                guid_id => $dup->{guid_id},
+                generator => $dup->{generator},
+                method => $dup->{method},
+                argument => $dup->{argument},
+                content_type => $dup->{type},
+            );
+
+            delete $dup_urls->{$_->{url}};
+            next;
+        }
+
         if (!@new_url) {
             # Nothing to redirect to, so mark the old one as gone.
             db_update($db, url => $_->{id},
                 status => 'G',
             );
-            $gone_changed = 1;
-
-            # Also mark as gone any URLs which were redirects to this one.
-            my $n = db_update($db, url => { redirect_to_id => $_->{id} },
-                status => 'G',
-                redirect_to_id => undef,
-            );
-            $redirects_changed = 1 if $n && $n > 0;
+            $self->_update_rewrite(\%gone_changed, 'gone', $_->{url});
+            $url_deactivated{$_->{url}} = $_;
         }
         else {
             # Change it to a redirect, if there are any active URLs which
@@ -1303,12 +1426,23 @@ sub _update_urls_in_db_txn
 
             if (defined $best_match) {
                 # Set old URL to redirect.
+                # TODO - I don't think I'm being consistent about what the
+                # content type of a redirect URL means.  Is it just a left-over
+                # from the old inactive URL, or as here is it the type of
+                # the target URL?
+                assert(defined $best_match->{id}) if DEBUG;
                 db_update($db, url => $_->{id},
                     content_type => $best_match->{type},
                     status => 'R',
                     redirect_to_id => $best_match->{id},
                 );
-                $redirects_changed = 1;
+                $self->_update_rewrite(\%redirects_changed, 'redirect',
+                                       $_->{url});
+                $url_changed{$best_match->{url}} = {
+                    %$best_match,
+                    old_url_info => $_,
+                };
+                delete $url_activated{$best_match->{url}};
 
                 # Adjust any which previously redirected to the old URL
                 # so that they point directly to the new one.
@@ -1318,20 +1452,34 @@ sub _update_urls_in_db_txn
                 );
             }
             else {
-                # Kill the old URL and any which redirect to it.
+                # Kill the old URL.
                 db_update($db, url => $_->{id},
                     status => 'G',
                 );
-                db_update($db, url => { redirect_to_id => $_->{id} },
-                    status => 'G',
-                    redirect_to_id => undef,
-                );
-                $gone_changed = 1;
+                $self->_update_rewrite(\%gone_changed, 'gone', $_->{url});
+                $url_deactivated{$_->{url}} = $_;
             }
         }
     }
 
-    return ($redirects_changed, $gone_changed);
+    return {
+        update_redirect_maps => \%redirects_changed,
+        update_gone_maps => \%gone_changed,
+        url_activated => \%url_activated,
+        url_deactivated => \%url_deactivated,
+        url_changed => \%url_changed,
+    };
+}
+
+sub _update_rewrite
+{
+    my ($self, $which_changed, $map_name, $url) = @_;
+    my ($config) = $self->{cms}->output_config($url);
+    return unless defined $config;
+
+    my $map_file = $config->{"${map_name}_map"};
+    $which_changed->{$map_file} = $config
+        if defined $map_file && !exists $which_changed->{$map_file};
 }
 
 =back
